import os
from string import Template
from zipfile import ZipFile

import discord
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_core.logging import LoggingSettings
from cpl_core.time import TimeFormatSettings
from cpl_discord.command import DiscordCommandABC
from cpl_query.extension import List
from cpl_translation import TranslatePipe
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.custom_file_logger_abc import CustomFileLoggerABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.helper.command_checks import CommandChecks
from bot_core.logging.command_logger import CommandLogger
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class LogCommand(DiscordCommandABC):

    def __init__(
            self,
            logger: CommandLogger,
            logging_settings: LoggingSettings,
            services: ServiceProviderABC,
            message_service: MessageServiceABC,
            client_utils: ClientUtilsServiceABC,
            translate: TranslatePipe,
            permissions: PermissionServiceABC,
            time_format: TimeFormatSettings,
            env: ApplicationEnvironmentABC
    ):
        DiscordCommandABC.__init__(self)

        self._logger = logger
        self._logging_settings = logging_settings
        self._services = services
        self._message_service = message_service
        self._client_utils = client_utils
        self._t = translate
        self._permissions = permissions

        self._env = env
        self._log_settings: LoggingSettings = logging_settings
        self._time_format_settings: TimeFormatSettings = time_format

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    def _reduce_path(self, p: str) -> str:
        if len(p.split('/')) == 1 or p == '':
            return p

        return self._reduce_path(os.path.dirname(p))

    @commands.hybrid_command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_technician()
    async def log(self, ctx: Context):
        self._logger.debug(__name__, f'Received command log {ctx}')

        possible_log_paths = List(str)
        possible_log_paths.append(self._reduce_path(self._logging_settings.path))

        file_extensions = List(str)
        if '.' in self._logging_settings.filename:
            split_filename = self._logging_settings.filename.split(".")
            file_extensions.append(f'.{split_filename[len(split_filename) - 1]}')

        for subclass in CustomFileLoggerABC.__subclasses__():
            logger: CustomFileLoggerABC = self._services.get_service(subclass)
            if logger is None:
                continue

            path = self._reduce_path(logger.settings.path)
            if '.' in logger.settings.filename:
                split_filename = logger.settings.filename.split(".")
                file_extension = f'.{split_filename[len(split_filename) - 1]}'
                if file_extension not in file_extensions:
                    file_extensions.append(file_extension)

            if path in possible_log_paths:
                continue
            possible_log_paths.append(path)

        files_str = "\n\t".join(possible_log_paths.to_list())
        self._logger.debug(__name__, f'Possible log files: \n\t{files_str}')

        files = List(str)
        for possible_path in possible_log_paths:
            for r, d, f in os.walk(possible_path):
                for file in f:
                    if '.' not in file:
                        continue

                    split_filename = file.split(".")
                    if f'.{split_filename[len(split_filename) - 1]}' not in file_extensions:
                        continue

                    files.append(os.path.join(r, file))

        files_str = "\n\t".join(files.to_list())
        self._logger.debug(__name__, f'Log files: \n\t{files_str}')

        zip_file = ZipFile('logs.zip', 'w')
        files.for_each(lambda x: zip_file.write(x))
        zip_file.close()
        await self._message_service.send_interaction_msg(ctx.interaction, self._t.transform('modules.technician.log_message'), file=discord.File(zip_file.filename, 'logs.zip'), ephemeral=True)
        os.remove(zip_file.filename)

        self._logger.trace(__name__, f'Finished log command')
